<?php

namespace Wp618\Com\CommingSoon;

if ( !defined( 'ABSPATH' ) ) exit;

//
class Setup
{
    //
    private $app;

    //
    public function __construct(){
        $this->app = App::getInstance();
    }

    //
    public function load_setup(){
        if($this->app->get_is_actived()){
            $option = 'general';
            $data = $this->get_setup_data($option);
            $this->app->display('general', $data);
        }else{
            $action = get_action('app_active');
            $data = [
                'action' => $action,
                'nonce' => \wp_create_nonce($action),
                'option' => 'none',
            ];

            $this->app->display('active', $data);
        }
    }

    //
    private function add_terms(){
        $taxonomy = $this->app->request->post('taxonomy', 'striptags', '');
        $txt = $this->app->request->post('txt', 'striptags', '');

        if(!in_array($taxonomy, ['category', 'post_tag', 'product_cat'])){
            $taxonomy = 'category';
        }

        $arr = explode("\n", $txt);

        $result = [];
        $num = 1;

        foreach ($arr as $line){
            $line = trim($line);
            if($line == ''){
                continue;
            }

            $category_id = $this->get_import_category_id($line, $taxonomy);

            if($category_id > 0){
                $result[] = ($num++) . '、' . $line;
            }
        }

        echo implode('<hr>', $result);
    }

    // 格式: 大类 | 小类
    private function get_import_category_id($name, $taxonomy){
        $arr = explode('|', $name);

        if($taxonomy == 'post_tag'){
            $arr = [$name];
        }

        $parent = 0;
        $term_id = 0;

        foreach ($arr as $term_name){
            $term_name = trim($term_name);
            if(strlen($term_name) < 1){
                continue;
            }

            $term = get_term_by('name', $term_name, $taxonomy);

            if(empty($term)){ // 写入
                $args = [];
                if($parent > 0){
                    $args = ['parent' => $parent];
                }

                $result = wp_insert_term($term_name, $taxonomy, $args);

                if (!is_wp_error($result)) {
                    $parent = (int) $result['term_id'];
                    $term_id = (int) $result['term_id'];
                }

                continue;
            }

            $parent = $term->term_id;
            $term_id = (int) $term->term_id;
        }

        return $term_id;
    }

    //
    public function apply_baidu_ai(){
        $option = 'general';
        $data = $this->get_setup_data($option);
        $data['action'] = get_action('get_ai_content');
        $this->app->display('baidu', $data);
    }

    const BAIDU_ACCESS_TOKEN_URL = 'https://aip.baidubce.com/oauth/2.0/token?';
    const BAIDU_ACCESS_ARTICLE_URL = 'https://aip.baidubce.com/rpc/2.0/ai_custom/v1/wenxinworkshop/chat/';

    //
    public function cb_get_ai_content(){
        $prompt = $this->app->request->post('prompt', 'striptags', '');

        $config = get_app_config();

        $client_id = isset($config['general']['client_id']) ? $config['general']['client_id'] : '';
        $token = $this->get_access_token($client_id);

        if(empty($client_id)){
            $content = '请输入百度申请的 CLIENT_ID 和 CLIENT_SECRET';
        }else if(empty($prompt)){
            $content = '请输入提示词';
        }else if(empty($token)){
            $content = '无效 token';
        }else{
            $content = $this->generateArticle($token, $prompt);
        }

        wp_send_json_success([
            'content' => $content,
        ]);
    }

    //
    private function get_access_token($client_id){
        $config = get_app_config();

        if(!empty($config['token']['time'])){
            if($config['token']['time'] > time()){
                return $config['token']['value'];
            }
        }

        $client_secret = isset($config['general']['client_secret']) ? $config['general']['client_secret'] : '';

        $data = [
            'grant_type' => 'client_credentials',
            'client_id' => $client_id,
            'client_secret' => $client_secret,
        ];

        $url = self::BAIDU_ACCESS_TOKEN_URL . http_build_query($data);

        $response = wp_remote_get($url);
        $body = wp_remote_retrieve_body($response);

        $json = json_decode($body, true);

        $token = '';
        if(isset($json['access_token'])){
            $token = $json['access_token'];

            $config['token'] = [
                'time' => time() + 86400,
                'value' => $token,
            ];

            set_app_config($config);
        }

        return $token;
    }

    //
    private function generateArticle($token, $prompt)
    {
        $config = get_app_config();
        $model = isset($config['general']['model']) ? $config['general']['model'] : '';
        $url = self::BAIDU_ACCESS_ARTICLE_URL . $model . '?access_token=' . $token;

        $headers = [
            'Content-Type: application/json'
        ];
        $data = [
            'messages' => [
                [
                    'role' => 'user',
                    'content' => $prompt
                ]
            ]
        ];

        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false); // 跳过证书检查
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false); // 从证书中检查SSL加密算法是否存在

        $response = curl_exec($ch);

        if (curl_errno($ch)) {
            echo 'Error 123:' . curl_error($ch);
            return null;
        }

        curl_close($ch);

        $responseData = json_decode($response, true);

        if (isset($responseData['result'])) {
            return $responseData['result'];
        } else {
            return "Error 456: " . $responseData['error_msg'];
        }
    }

    //
    private function get_setup_data($option){
        $config = get_app_config();
        $set = $this->get_render_items($option, $config);

        $action = get_action('setup_save');

        $data = [
            'action' => $action,
            'nonce' => \wp_create_nonce($action),
            'option' => $option,
            'set' => $set,
            'page' => $this->app->request->get('page', 'wdash'),
        ];

        return $data;
    }

    //
    private function get_render_items($option, $config){
        if(isset($config[$option])){
            return $config[$option];
        }

        $list = [];

        if(isset(Constant::SETUP_FIELDS[$option])){
            foreach (Constant::SETUP_FIELDS[$option] as $name => $val){
                $list[$name] = $val[1];
            }
        }

        return $list;
    }
}